             .title        "NTDEC TC-112 Reader Plugin"


;10/21/2010 
;Written by BootGod
;Version 1.0

;IMPORTANT! If your dumping a cart with a 28-pin CHR ROM and an 74LS00, it
;most likely will not dump the CHR correctly. No known solution exists other
;than removing the CHR ROM and reading it via a programmer, etc.
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh


address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
compaddr:    .equ 00e4h
compaddr_lo: .equ 00e4h
compaddr_hi: .equ 00e5h

curbank:    .equ 00e2h
banksleft:  .equ 00e3h
romsize:    .equ 00e6h

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "NTDEC TC-112 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 64 >> 4,256 >> 4
             .db 128 >> 3,256 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here
 
;determine H or V mirroring
             ldx #00h       
             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne horz_mir
             lda 2007h
             cmp #0aah
             bne horz_mir
             lda #020h
             jsr wr_ppu
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #0aah
             bne horz_mir
             inx		;vertical

horz_mir:    txa
             jsr send_byte	;send mirroring byte

;Check PRG size
             lda #1		;1 = 64K
             sta romsize
             lda #7		;compare bank 7 to fixed last bank
             jsr comp_pbank
             beq got_prg
             asl romsize	;2 = 128K
             lda #0fh		;compare bank 15 to fixed last bank
             jsr comp_pbank
             beq got_prg
             asl romsize	;4 = 256K

;Send PRG header
got_prg:     lda #0
             jsr send_byte
             lda romsize
             jsr send_byte
             lda #01h
             jsr send_byte

             lda #0
             sta curbank

             lda romsize	;convert to # of 8K banks
             asl a
             asl a
             asl a
             sta banksleft

;Send current 8K bank
send_pbank:  lda curbank
             jsr wr_pbank
             inc curbank

             lda #0
             sta address_lo
             lda #080h
             sta address_hi
             
             ldy #0
             ldx #32

send_pblk:   lda (address),y
             jsr send_byte
             iny
             bne send_pblk
             inc address_hi
             dex
             bne send_pblk
             dec banksleft
             bne send_pbank

;Check if CHR is 128K or 256K; Upper half will be open bus if only 128K

             lda #4		;init as 256K
             sta romsize
             lda #0ffh		;load a CHR bank in upper half
             jsr wr_cbank
             inc a
             sta address	;set address to PPU $0000
             jsr wr_ppu
             lda 2007h
obcheck:     lda 2007h		;if CHR open bus, will read 00,01,02...FF
             cmp address
             bne chr128
             inc address
             bne obcheck

             jmp got_chr

chr128:      lsr romsize
             
;Send CHR header
      
got_chr:     lda #0
             jsr send_byte
             lda romsize
             jsr send_byte
             lda #2
             jsr send_byte

             lda romsize	;convert romsize to # of 4K banks
             asl a
             asl a
             asl a
             asl a
             sta banksleft
             lda #0
             sta curbank

send_cbank:  lda curbank
             jsr wr_cbank

             lda #00h
             jsr wr_ppu
             lda 2007h

             ldx #16		;set counters for 4K bank xfer
             ldy #0

send_cblk:   lda 2007h
             jsr send_byte
             iny
             bne send_cblk
             dex
             bne send_cblk
             inc curbank
             dec banksleft
             bne send_cbank	;4K has been sent, load next bank
             
;Send termination header

             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte

             rts

;Loads bank # in A to $8000 and compares to $E000

comp_pbank:  jsr wr_pbank
             lda #00h
             sta address_lo
             sta compaddr_lo

             lda #080h
             sta address_hi
             lda #0e0h
             sta compaddr_hi

             ldy #0

cpbank_l:    lda (address),y
             cmp (compaddr),y
             bne pdiff
             iny
             bne cpbank_l
             inc address_hi
             inc compaddr_hi
             bne cpbank_l

pdiff:       rts

;Bank selection

wr_pbank:    sta 6003h
             rts

wr_cbank:    asl a
             asl a
             sta 6000h
             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
